"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("../../styles");
var React = require("react");
var react_codemirror2_1 = require("react-codemirror2");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var common_1 = require("../../common");
if (common_1.is.browser) {
    require('codemirror/mode/mathematica/mathematica.js');
    require('codemirror/mode/javascript/javascript.js');
}
var FormulaInput = (function (_super) {
    tslib_1.__extends(FormulaInput, _super);
    function FormulaInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isLoaded: false };
        _this.unmounted$ = new rxjs_1.Subject();
        _this.state$ = new rxjs_1.Subject();
        _this.editorRef = function (ref) { return (_this.editor = ref && ref.editor); };
        _this._events$ = new rxjs_1.Subject();
        _this.events$ = _this._events$.pipe(operators_1.takeUntil(_this.unmounted$), operators_1.share());
        _this.fire = function (e) { return _this._events$.next(e); };
        _this.focusHandler = function (isFocused) {
            return function () {
                if (isFocused) {
                    _this.fire({ type: 'INPUT/formula/focus', payload: {} });
                }
                if (!isFocused) {
                    _this.fire({ type: 'INPUT/formula/blur', payload: {} });
                }
            };
        };
        _this.handleBeforeChange = function (editor, data, value) {
            var _a = _this.props, _b = _a.isMultiLine, isMultiLine = _b === void 0 ? false : _b, maxLength = _a.maxLength;
            var change = data;
            var char = change.text[0];
            if (char.includes('\t')) {
                var isCancelled_1 = false;
                _this.fire({
                    type: 'INPUT/formula/tab',
                    payload: {
                        cancel: function () { return (isCancelled_1 = true); },
                        get isCancelled() {
                            return isCancelled_1;
                        },
                    },
                });
                if (isCancelled_1) {
                    return;
                }
            }
            if (!isMultiLine) {
                var wasCancelled = singleLineOnly(change);
                if (wasCancelled) {
                    return;
                }
            }
            if (maxLength !== undefined) {
                var clippedValue = formatValue(value, { maxLength: maxLength, isMultiLine: isMultiLine });
                if (clippedValue !== value) {
                    if (change.update && change.origin === '+input') {
                        change.update(undefined, undefined, [clippedValue]);
                        value = clippedValue;
                    }
                }
            }
            var from = _this.props.value || '';
            var to = value;
            var isMax = maxLength === undefined ? null : to.length === maxLength;
            _this.fire({ type: 'INPUT/formula/change', payload: { from: from, to: to, isMax: isMax, char: char } });
        };
        return _this;
    }
    FormulaInput.prototype.componentWillMount = function () {
        var _this = this;
        var events$ = this.props.events$;
        this.state$.pipe(operators_1.takeUntil(this.unmounted$)).subscribe(function (e) { return _this.setState(e); });
        if (events$) {
            this.events$.subscribe(function (e) { return events$.next(e); });
        }
    };
    FormulaInput.prototype.componentDidMount = function () {
        var _this = this;
        var _a = this.props, focusOnLoad = _a.focusOnLoad, selectOnLoad = _a.selectOnLoad;
        common_1.time.delay(0, function () {
            _this.setState({ isLoaded: true }, function () {
                if (focusOnLoad) {
                    _this.focus();
                }
                if (selectOnLoad) {
                    _this.selectAll();
                }
            });
        });
    };
    FormulaInput.prototype.componentWillUnmount = function () {
        this.unmounted$.next();
        this.unmounted$.complete();
    };
    Object.defineProperty(FormulaInput.prototype, "isFocused", {
        get: function () {
            return this.editor ? this.editor.hasFocus() : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormulaInput.prototype, "height", {
        get: function () {
            var _a = this.props, _b = _a.isMultiLine, isMultiLine = _b === void 0 ? false : _b, height = _a.height;
            if (height !== undefined) {
                return height;
            }
            if (!isMultiLine && height === undefined) {
                return 21;
            }
            return;
        },
        enumerable: true,
        configurable: true
    });
    FormulaInput.prototype.focus = function () {
        if (this.editor) {
            this.editor.focus();
        }
    };
    FormulaInput.prototype.redraw = function () {
        this.forceUpdate();
        if (this.editor) {
            this.editor.refresh();
        }
    };
    FormulaInput.prototype.selectAll = function () {
        this.editor.execCommand('selectAll');
    };
    FormulaInput.prototype.render = function () {
        var _a = this.props, _b = _a.isMultiLine, isMultiLine = _b === void 0 ? false : _b, maxLength = _a.maxLength, _c = _a.mode, mode = _c === void 0 ? 'mathematica' : _c;
        var height = this.height;
        var value = formatValue(this.props.value, { maxLength: maxLength, isMultiLine: isMultiLine });
        var styles = {
            base: common_1.css({
                height: height,
                Scroll: isMultiLine,
                overflow: 'hidden',
                lineHeight: 1,
            }),
        };
        var el = this.state.isLoaded && (React.createElement(react_codemirror2_1.Controlled, { ref: this.editorRef, options: { mode: mode }, value: value, onBeforeChange: this.handleBeforeChange, onFocus: this.focusHandler(true), onBlur: this.focusHandler(false) }));
        return (React.createElement("div", tslib_1.__assign({ className: 'FormulaInput' }, common_1.css(styles.base, this.props.style)), el));
    };
    return FormulaInput;
}(React.PureComponent));
exports.FormulaInput = FormulaInput;
function singleLineOnly(change) {
    var typedNewLine = change.origin === '+input' && typeof change.text === 'object' && change.text.join('') === '';
    if (typedNewLine) {
        return change.cancel();
    }
    var pastedNewLine = change.origin === 'paste' && typeof change.text === 'object' && change.text.length > 1;
    if (pastedNewLine && change.update) {
        var newText = change.text.join(' ');
        return change.update(undefined, undefined, [newText]);
    }
    return null;
}
function formatValue(value, options) {
    var maxLength = options.maxLength, _a = options.isMultiLine, isMultiLine = _a === void 0 ? false : _a;
    value = value || '';
    if (!isMultiLine) {
        value = value.replace(/\n/g, '');
    }
    if (maxLength !== undefined && value.length > maxLength) {
        value = value.substr(0, maxLength);
    }
    return value;
}
